# Boots Client Project
# www.launchpad.net/boots
#
# ##### BEGIN LICENSE BLOCK #####
#
# Copyright (C) 2009-2010 Clark Boylan, Ken Brotherton, Max Goodman,
#                         Victoria Lewis, David Rosenbaum, and Andreas Turriff
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ##### END LICENSE BLOCK #####

import exceptions
import traceback

from boots.api import _, n_

class BootsError(exceptions.StandardError):
    """Base class of boots error exceptions."""
    pass

class BootsWarning(exceptions.Warning):
    """Base class of boots warnings."""
    pass

class ConnectionError(BootsError):
    """Errors related to server connections."""
    pass

class LostConnectionError(ConnectionError):
    """Raised when a connection is lost."""
    def __init__(self, msg=None, server=None, reconnect_exception=None):
        self.msg = msg
        self.server = server
        self.reconnect_exception = reconnect_exception
    
    def __str__(self):
        msg = []
        if self.msg:
            msg.append(self.msg)
        else:
            msg.append(_("Lost connection to {0}").format(self.server))

        if self.reconnect_exception:
            exc_lines = traceback.format_exception_only(type(self.reconnect_exception), self.reconnect_exception)
            exc_text = ". ".join(line.strip() for line in exc_lines)
            msg.append(_("(Exception upon reconnect: {0})").format(exc_text))
            
        return " ".join(msg)
