# Boots Client Project
# www.launchpad.net/boots
#
# ##### BEGIN LICENSE BLOCK #####
#
# Copyright (C) 2009-2010 Clark Boylan, Ken Brotherton, Max Goodman,
#                         Victoria Lewis, David Rosenbaum, and Andreas Turriff
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ##### END LICENSE BLOCK #####

from weakref import ref, ReferenceType

class Hooks(dict):
    def __init__(self, *names):
        dict.__init__(self, ((name, Hook(name)) for name in names))

class Hook(object):
    def __init__(self, name):
        self.name = name
        self.callbacks = set()
    
    def register(self, callback, weak=False):
        if weak:
            callback = ref(callback)
        self.callbacks.add(callback)
        return callback
    
    def unregister(self, callback):
        self.callbacks.remove(callback)
    
    def call(self, *args, **kwargs):
        dead_refs = set()
        for callback in self.callbacks:
            if isinstance(callback, ReferenceType):
                callback = callback()
            
            if callback:
                callback(*args, **kwargs)
            else:
                dead_refs.add(callback)
        self.callbacks -= dead_refs
                
