# Boots Client Project
# www.launchpad.net/boots
#
# ##### BEGIN LICENSE BLOCK #####
#
# Copyright (C) 2009-2010 Clark Boylan, Ken Brotherton, Max Goodman,
#                         Victoria Lewis, David Rosenbaum, and Andreas Turriff
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ##### END LICENSE BLOCK #####

from boots.lib.lingos import lingo
import subprocess
import threading

class ExternalInterpreter(lingo.Interpreter):
    """Provides an interface that can be used to allow any external interpreter to be used as a
    boots lingo."""
    def __init__(self, executable, executable_args, *args, **kwargs):
        """Create a new interpreter using the specified executable with the specified arguments."""
        super(ExternalInterpreter, self).__init__(*args, **kwargs)
        self.executable = executable
        self.executable_args = executable_args
        self.interpreter = None

    def _readlines(self):
        while True:
            # Remove the newline from the end of each line.
            line = self.interpreter.stdout.readline()[:-1]

            with self.lock:
                self.output_lines += [line]
    
    def evaluate(self, code):
        """Evaluate code using the external interpreter.  All return values are strings."""
        if self.interpreter == None:
            self.interpreter = subprocess.Popen(self.executable, *self.executable_args, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.STDOUT)
            self.output_lines = []
            self.lock = threading.RLock()
            # Since self.interpreter.stdout.readline could block, a thread is used to read the output.
            self.read_thread = threading.Thread(target = self._readlines)
            self.read_thread.daemon = True
            self.read_thread.start()
            # The maximum amount of time to wait for output.
            # FIXME: This is a kludge and should be fixed if possible.

        read_wait = 0.1
        
        with self.lock:
            # Every line must be postfixed with a newline or the input will not be received.
            self.interpreter.stdin.write(code + '\n')
            self.interpreter.stdin.flush()

        self.read_thread.join(read_wait)

        with self.lock:
            output_lines = self.output_lines
            self.output_lines = []

        # Restart the reader thread if required.
        if not self.read_thread.is_alive():
            self.read_thread = threading.Thread(target = self._readlines)
            self.read_thread.start()
        
        return output_lines
