# Boots Client Project
# www.launchpad.net/boots
#
# ##### BEGIN LICENSE BLOCK #####
#
# Copyright (C) 2009-2010 Clark Boylan, Ken Brotherton, Max Goodman,
#                         Victoria Lewis, David Rosenbaum, and Andreas Turriff
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ##### END LICENSE BLOCK #####

from lexer import tokens
import ply.yacc as yacc

from boots.lib.lingos.lisp import builtins
from boots.lib.lingos.lisp import objects
from boots.lib import _, n_


def p_expression_integer(p):
    'expression : INTEGER'
    p[0] = objects.Integer(int(p[1]))

def p_expression_string(p):
    'expression : STRING'
    p[0] = objects.String(p[1][1:len(p[1]) - 1])

def p_expression_symbol(p):
    'expression : SYMBOL'
    p[0] = objects.Symbol(p[1])

def p_expression_quoted(p):
    'expression : quoted_expression'
    p[0] = p[1]

def p_expression(p):
    'expression : LEFT_PAREN expression_list RIGHT_PAREN'
    p[0] = p[2]

def p_quoted_expression(p):
    'quoted_expression : QUOTE expression'
    p[0] = objects.Quote(p[2])

def p_expression_list_empty(p):
    'expression_list : '
    p[0] = builtins.nil

def p_expression_list(p):
    'expression_list : expression expression_list'
    p[0] = objects.Cons(p[1], p[2])

def p_error(p):
    raise SyntaxError(_('syntax error'))

# Disable debugging output and caching of parse tables.
parser = yacc.yacc(debug = 0, write_tables = 0)
