# Boots Client Project
# www.launchpad.net/boots
#
# ##### BEGIN LICENSE BLOCK #####
#
# Copyright (C) 2009-2010 Clark Boylan, Ken Brotherton, Max Goodman,
#                         Victoria Lewis, David Rosenbaum, and Andreas Turriff
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ##### END LICENSE BLOCK #####

import code
import traceback

from boots.api import errors
from boots.lib.lingos import lingo
from boots.lib.ui.components.help import HelpTopic
from boots.lib import _, n_

class PythonInterpreter(lingo.Interpreter):
    """Implements a python interpreter that will send commands to the python interpreter that runs
    boots."""
    
    help = HelpTopic("python", _("Python Lingo"), _("Integrated Python interpreter using the Boots API."))    
    
    def __init__(self, *args, **kwargs):
        super(PythonInterpreter, self).__init__(*args, **kwargs)
        # These dictionaries are used to store the variables.
        self._globals = {"console" : self.console}
        self._locals = {}
    
    def input_complete(self, command):
        # Consider input complete if the last line did not end with ":", the last two characters
        # were newlines or it was empty.
        if command.endswith("\n"):
            command = command[:-1]
        try:
            if code.compile_command(command):
                return command
            else:
                return None
        except (SyntaxError, OverflowError, ValueError):
            return command

    def evaluate(self, code):
        if code.strip():
            try:
                return eval(code, self._globals, self._locals)
            # If eval results in an error, it might be because code is a statement rather than an expression.
            except:
                try:
                    exec code in self._globals, self._locals
                except Exception:
                    return errors.BootsError(traceback.format_exc())
                
    def present(self, obj):
        if not isinstance(obj, Exception) and obj is not None:
            return repr(obj)
        else:
            return obj
                    
lingo.register('python', PythonInterpreter)
