# Boots Client Project
# www.launchpad.net/boots
#
# ##### BEGIN LICENSE BLOCK #####
#
# Copyright (C) 2009-2010 Clark Boylan, Ken Brotherton, Max Goodman,
#                         Victoria Lewis, David Rosenbaum, and Andreas Turriff
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ##### END LICENSE BLOCK #####

from boots.api.nodes.node import NodeGraph, SyncNode, IteratorNode
from boots.lib.ui.components.help import HelpTopic
from boots.lib.lingos import lingo
from boots.lib import _, n_

class SQLInterpreter(lingo.Interpreter):
    help = HelpTopic("sql", _("SQL Lingo"), _("Raw SQL commands sent to a server."))
    
    def __init__(self, *args, **kwargs):
        super(SQLInterpreter, self).__init__(*args, **kwargs)
        self.terminating_char = self.console.config["terminating_char"]

    def input_complete(self, command):
        try:
            # FIXME ignore leading whitespace when testing prefixes.
            # Comments should be removed when in the middle of a statement
            # as well.
            if command.startswith("\\") or command.startswith("-- ") or \
               self.terminating_char == "\n":
                return command
            # FIXME ignore trailing whitespace when testing terminators.
            elif command.strip("\n").endswith(self.terminating_char):
                return command[0:-len(self.terminating_char)]
            else:
                return None
        except IndexError:
            return command

    def evaluate(self, code):
        if code.strip():
            cmd_queue = IteratorNode([code])
            nodes = NodeGraph("sql")
            nodes.add(cmd_queue)
            for server in self.console.servers:
                server_node = SyncNode(server.execute)
                nodes.add_output(server_node)
                cmd_queue.connect_output(server_node)
            return nodes

lingo.register('sql', SQLInterpreter)
