# Boots Client Project
# www.launchpad.net/boots
#
# ##### BEGIN LICENSE BLOCK #####
#
# Copyright (C) 2009-2010 Clark Boylan, Ken Brotherton, Max Goodman,
#                         Victoria Lewis, David Rosenbaum, and Andreas Turriff
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ##### END LICENSE BLOCK #####

"""This module contains the base test classes to be used for boots unit
tests. BootsBaseTester contains the test config. BootsQueryTester includes
the config and its setUp method will load a fresh test DB for testing."""

import unittest
import os.path
from subprocess import Popen, PIPE

class BootsBaseTester(unittest.TestCase):
    """Simple unit test test case class that includes a boots testing config.

    All boots unit test classes should inherit from this class or one of its
    children."""
    config = {"host": "localhost",
              "port": 9306,
              "database": "boots_fec_test"}

class BootsQueryTester(BootsBaseTester):
    """Unit test class that includes a setUp method that can be used to load
    a fresh copy of the test database before each unit test.

    Do not use this class unless you need a fresh database loaded into the
    server before each test. The time cost of doing this is quite high and
    should be avoided whenever possible. Additionally the current drizzle
    client should be in your path when you run tests using this class."""
    def setUp(self):
        """Default setUp method that loads a fresh copy of the test DB into
        the server before each test.

        This allows any classes deriving from this one to easily get clean test
        data for its tests."""
        # The drizzle client should be in your path allowing the DB to be
        # created if necessary.
        
        base_tests_path = os.path.dirname(os.path.realpath(__file__))
        fec_sql_path = os.path.join(base_tests_path, "fec.sql")
        boots_path = os.path.join(base_tests_path, "../../bin/boots")
        
        cmd = "{boots} -H {host} -p {port} -f {file}".format(boots=boots_path,
                                                             file=fec_sql_path, 
                                                             **self.config)
        p = Popen(cmd, shell=True)
        p.communicate()
