#!/usr/bin/env python
# Boots Client Project
# www.launchpad.net/boots
#
# ##### BEGIN LICENSE BLOCK #####
#
# Copyright (C) 2009-2010 Clark Boylan, Ken Brotherton, Max Goodman,
#                         Victoria Lewis, David Rosenbaum, and Andreas Turriff
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ##### END LICENSE BLOCK #####

"""This module contains the boots test driver. The test driver runs all tests by
default or any specified on the command line."""

import unittest
from optparse import OptionParser
import sys
import os
import os.path
from boots_unit_test import BootsBaseTester

# Path mangling to import boots files properly.
new_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), "../..")
sys.path.insert(0, new_path)
from api import api_tests
from api.nodes import node_tests
from app import app_tests
from lib import console_tests
from lib.lingos import lingo_tests
from lib.ui import ui_tests

def boots_test_driver():
    """Function that reads test options from the command line and runs the
    appropriate tests."""
    parser = OptionParser()
    parser.add_option("-H", "--host", dest="host",
                      default=BootsBaseTester.config["host"],
                      help="Host running the Drizzle server on which the test\
                            DB should be run.")
    parser.add_option("-p", "--port", type="int", dest="port",
                      default=BootsBaseTester.config["port"],
                      help="Port the Drizzle server is running on.")
    parser.add_option("-D", "--database", dest="database",
                      default=BootsBaseTester.config["database"],
                      help="Database to use for testing on the Drizzle server.")
    (options, args) = parser.parse_args()
    BootsBaseTester.config.update(vars(options))
    if not args:
        modules = [api_tests, node_tests, app_tests,
                   console_tests, lingo_tests, ui_tests]
    else:
        modules = map(globals().__getitem__, args)
    for module in modules:
        suite = unittest.TestLoader().loadTestsFromModule(module)
        unittest.TextTestRunner(verbosity=2).run(suite)

if __name__ == "__main__":
    boots_test_driver()
